package cz.jzitnik.game;

import cz.jzitnik.api.ApiService;
import cz.jzitnik.api.types.Item;
import cz.jzitnik.utils.Cli;
import cz.jzitnik.utils.ConfigPathProvider;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
public class CommandPalette {
    public enum CommandType {
        CUSTOM,
        INVENTORY_OPEN,
        USER_PROFILE,
        CHAT_OPEN,
        CHAT_SEND,
        PLAYER_LIST_OPEN,
        DROP_ITEM,
        CHECK_WINNABLE,
        EXIT,
    }

    @Getter
    @Setter
    @AllArgsConstructor
    public static class Command {
        private String name;
        private CommandType commandType;

        public Command(String name) {
            this.name = name;
            this.commandType = CommandType.CUSTOM;
        }

        @Override
        public String toString() {
            if (commandType == CommandType.CUSTOM) {
                return name;
            }

            return Cli.Colors.BLUE + name + Cli.Colors.RESET;
        }
    }

    private List<Command> commands;
    private ApiService apiService;
    private String playerKey;

    public CommandPalette(List<String> commands, ApiService apiService, String playerKey) {
        this.commands = commands.stream().map(Command::new).toList();
        this.apiService = apiService;
        this.playerKey = playerKey;
    }

    public int displayIndex() throws IOException {
        var res = apiService.gameWon(playerKey).execute();
        var content = res.body().getData().get();

        if (content.isWon()) {
            System.out.println("\n\n");
            Cli.printHeader("Hra byla vyhrána");
            Cli.typeSkritek(content.getMsg());
            Cli.printHeader("Konec");

            var configPath = ConfigPathProvider.getPath();

            File file = new File(Path.of(configPath, "config.json").toString());
            file.delete();

            System.exit(0);
        }

        List<Command> allCommands = new ArrayList<>(commands);

        // Commands that can be used anywhere
        allCommands.add(new Command("Otevřít inventář", CommandType.INVENTORY_OPEN));
        allCommands.add(new Command("Otevřít profil uživatele", CommandType.USER_PROFILE));
        allCommands.add(new Command("Otevřít chat", CommandType.CHAT_OPEN));
        allCommands.add(new Command("Odeslat zprávu do chatu", CommandType.CHAT_SEND));
        allCommands.add(new Command("Zobrazit list všech hráčů", CommandType.PLAYER_LIST_OPEN));
        allCommands.add(new Command("Vyhodit item z inventáře", CommandType.DROP_ITEM));
        allCommands.add(new Command("Zkontrolovat vyhratelnost hry", CommandType.CHECK_WINNABLE));
        allCommands.add(new Command("Odejít ze hry", CommandType.EXIT));

        var responseInvFull = apiService.inventoryIsFull(playerKey).execute();
        var inv_full = responseInvFull.body().getData().get().isFull();

        System.out.println("\n");

        if (inv_full) {
            System.out.println(Cli.Colors.RED + "Váš inventář je plný! Jakýkoliv předmět, který dostanete, bude ztracen!" + Cli.Colors.RESET);
        }
        var selectedOption = Cli.selectOptionIndex(allCommands.stream().map(Command::toString).toList());

        var command = allCommands.get(selectedOption);

        return switch (command.getCommandType()) {
            case CUSTOM -> selectedOption;
            case EXIT -> {
                Cli.info("Odpojuji se ze hry...");
                System.exit(0);

                yield 0;
            }
            case INVENTORY_OPEN -> {
                var response = apiService.getMe(playerKey).execute();
                assert response.body() != null;
                var body = response.body().getData();
                var me = body.get();

                var inventory = new Inventory(me, apiService, playerKey);
                inventory.display();

                yield displayIndex();
            }
            case USER_PROFILE -> {
                var response = apiService.getMe(playerKey).execute();
                assert response.body() != null;
                var body = response.body().getData();
                var me = body.get();

                System.out.println("\n\n" + me);

                yield displayIndex();
            }
            case CHAT_OPEN -> {
                var chat = new Chat(apiService, playerKey);
                chat.display();

                yield displayIndex();
            }
            case CHAT_SEND -> {
                var chat = new Chat(apiService, playerKey);
                chat.sendMessage();

                yield displayIndex();
            }
            case CHECK_WINNABLE -> {
                var response = apiService.winnable(playerKey).execute();
                var winnable = response.body().getData().get();

                System.out.println(Cli.Colors.RED + "Experimentální!!!" + Cli.Colors.RESET + " Toto není 100% jestli je hra vyhratelná.");
                if (winnable) {
                    Cli.info("Hra je dost možná vyhratelná!");
                } else {
                    Cli.info("Hra je dost možná nevyhratelná!");
                }

                yield displayIndex();
            }
            case PLAYER_LIST_OPEN -> {
                var playerList = new PlayerList(apiService, playerKey);
                playerList.display();

                yield displayIndex();
            }
            case DROP_ITEM -> {
                var response = apiService.getMe(playerKey).execute();
                var me = response.body().getData().get();
                var inventory = me.getInventory();

                if (inventory.isEmpty()) {
                    Cli.error("Máš prázdný inventář!");

                    yield displayIndex();
                }

                var options = inventory.stream().map(Item::toString).toList();
                var selectedOptionIndex = Cli.selectOptionIndex(options);

                var selectedItem = inventory.get(selectedOptionIndex);

                var response2 = apiService.dropItem(playerKey, selectedItem.getId()).execute();

                if (response2.code() != 200) {
                    Cli.error("Nastala chyba při vyhazování itemu!");
                } else {
                    Cli.success("Item byl vyhozen!");
                }

                yield displayIndex();
            }
        };
    }
}
